-- Handle all rendering related tasks

local M = {}

M.render = function(data,wd)
    local exe = M.executable()
    local config = rs.pathConcat(rs.pathConcat(rs.storageDirectory,"database"),"renderer.json")
    if settings.getInteger("ServerOS",-1) == 0 then
        rs.runExecutable(exe,{"/i","\""..data.layer.."\"","/c","\""..config.."\""})
    else
        rs.runExecutable(exe,{"-i",data.layer,"-c",config})
    end
    if data.list ~= "stl" then
        -- Check if it is gcode and still exists
        local printer = rs.printerBySlug(data.slug)
        local manager = printer.printjobManagerFromFile(data.layer)
        local job = manager.findById(data.id)
        if job == nil then
            manager.deleteAssociatedFilesForId(data.id)
        else
            rs.fireEvent("newRenderImage",data.slug,{list=data.list,id=data.id,slug=data.slug})
        end
    end
end

M.gcodeInfoUpdatedHandler = function(event, printer, data)
    local manager = printer:printjobManagerFromFile(data.modelPath)
    local name = manager:decodeNamePart(data.modelPath)
    local id = manager:decodeIdPart(data.modelPath)
    local imageFile = manager:encodeName(id, name .. "_l","png")
    if not rs.pathExists(imageFile) then
        rs.dispatchJob("rendering.render",{id=id, image=imageFile, layer=manager:encodeName(id,name,"layer"),slug=printer.slug,list=manager.managerType},1000)
    end
end

-- Find the executable of the renderer software
M.executable = function()
    local exe = rs.applicationDirectory
    local unixName = rs.pathConcat(exe,"RepetierRenderer")
    local winName = rs.pathConcat(exe,"RepetierRenderer.exe")
    if rs.pathExists(unixName) then return unixName end
    if rs.pathExists(winName) then return winName end
    return nil
end

-- Render all gcode files having no images yet
M.updateMissingIn = function(manager)
    local list = manager.listJobs()
    for _,model in pairs(list) do
        local layerFile = manager:encodeName(model.id, model.name,"layer")
        local imageFile = manager:encodeName(model.id, model.name .. "_l","png")
        if rs.pathExists(layerFile) and rs.pathExists(imageFile) == false then
            rs.dispatchJob("rendering.render",{id=model.id,layer=manager:encodeName(model.id,model.name,"layer"),slug=model.printer.slug,list=manager.managerType},900)
        end
    end

end

M.updateIn = function(manager)
    local list = manager.listJobs()
    for _,model in pairs(list) do
        local layerFile = manager:encodeName(model.id, model.name,"layer")
        local imageFile = manager:encodeName(model.id, model.name .. "_l","png")
        if rs.pathExists(layerFile) then
            rs.dispatchJob("rendering.render",{id=model.id,layer=manager:encodeName(model.id,model.name,"layer"),slug=model.printer.slug,list=manager.managerType},1)
        end
    end

end

-- Scan all printers for unrendered files
M.updateAllMissing = function()
    -- Copy configuration if not present
    local source = rs.pathConcat(rs.pathConcat(rs.pathConcat(rs.modulesDirectory,"server"),"data"),"renderer.json")
    local target = rs.pathConcat(rs.pathConcat(rs.storageDirectory,"database"),"renderer.json")
    if not rs.pathExists(target) then
        rs.pathCopy(source,target)
    end
    -- Render all gcodes
    local printers = rs.printerList
    for _,printer in pairs(printers) do
        M.updateMissingIn(printer.modelPrintjobManager)
        M.updateMissingIn(printer.jobPrintjobManager)
    end
end

M.getRenderSettings = function(printer, data, result)
    return rs.loadJSONFile(rs.pathConcat(rs.pathConcat(rs.storageDirectory,"database"),"renderer.json"))
end

M.setRenderSettings = function(printer, data, result)
    rs.saveJSONObjectFile(rs.pathConcat(rs.pathConcat(rs.storageDirectory,"database"),"renderer.json"),data)
    return result
end

M.resetRenderSettings = function(printer, data, result)
    local source = rs.pathConcat(rs.pathConcat(rs.pathConcat(rs.modulesDirectory,"server"),"data"),"renderer.json")
    local target = rs.pathConcat(rs.pathConcat(rs.storageDirectory,"database"),"renderer.json")
    rs.pathCopy(source,target)
    return rs.loadJSONFile(target)
end

M.updateAllRenderings = function(printer, data, result)
    local printers = rs.printerList
    for _,printer in pairs(printers) do
        M.updateIn(printer.modelPrintjobManager)
        M.updateIn(printer.jobPrintjobManager)
    end
end

M.modulePath = function()
    return rs.pathConcat(rs.modulesDirectory,"server")
end

M.getImage = function(request, response)
    local imgSize = request.param("t","l")
    local img = rs.pathConcat(rs.pathConcat(rs.pathConcat(M.modulePath(),"www"),"img"),"rendering_" .. imgSize .. ".png")
    local queue = request.param("q","models")
    local id = request.param("id","-1")
    local slug = request.param("slug","")
    local printer = rs.printerBySlug(slug)
    if(printer == nil) then
        print("Unkown printer " .. slug .. " for rendered image")
        response.viewStaticFile(img)
        return
    end
    local jobQueue
    if(queue == "models") then
        jobQueue = printer.modelPrintjobManager
    else
        jobQueue = printer.jobPrintjobManager
    end
    local job = jobQueue.findById(id)
    if(job == nil) then
        print("Unkown job id " .. id .. " for rendered image")
        response.viewStaticFile(img)
        return
    end
    local img2 = jobQueue.encodeName(id,job.name .. "_" .. imgSize,"png")
    if(rs.pathExists(img2)) then
        img = img2
    end
    response.viewStaticFile(img)
end

M.init = function()
    server:registerEventHandler("gcodeInfoUpdated", M.gcodeInfoUpdatedHandler)
    server:registerRunAtStartup(M.updateAllMissing)
    server:registerAction('getRenderSettings',M.getRenderSettings)
    server:registerAction('setRenderSettings',M.setRenderSettings)
    server:registerAction('resetRenderSettings',M.resetRenderSettings)
    server:registerAction('updateAllRenderings',M.updateAllRenderings)
    server:registerDynRequest('render_image',M.getImage)
end

return M;
